@@ -1,5 +1,5 @@
 /* defs.h -- data types and declarations.
-   Copyright (C) 1990, 91, 92, 93, 94, 2000, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1990, 91, 92, 93, 94, 2000, 2004, 2005 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -147,6 +147,7 @@ typedef int boolean;
 #endif
 
 struct predicate;
+struct options;
 
 /* Pointer to a predicate function. */
 typedef boolean (*PRED_FUNC)(char *pathname, struct stat *stat_buf, struct predicate *pred_ptr);
@@ -277,6 +278,22 @@ struct format_val
   struct quoting_options *quote_opts;
 };
 
+/* evaluation cost of a predicate */
+enum EvaluationCost
+{
+  NeedsNothing,
+  NeedsType,
+  NeedsStatInfo,
+  NeedsLinkName,
+  NeedsAccessInfo,
+  NeedsSyncDiskHit,
+  NeedsEventualExec,
+  NeedsImmediateExec,
+  NeedsUserInteraction,
+  NeedsUnknown,
+  NumEvaluationCosts
+};
+    
 struct predicate
 {
   /* Pointer to the function that implements this predicate.  */
@@ -308,8 +325,19 @@ struct predicate
   /* True if this predicate node requires knowledge of the file type. */
   boolean need_type;
 
+  enum EvaluationCost p_cost;
+
+  /* est_success_rate is a number between 0.0 and 1.0 */
+  float est_success_rate;
+  
   /* True if this predicate should display control characters literally */
   boolean literal_control_chars;
+
+  /* True if this predicate didn't originate from the user. */
+  boolean artificial;
+
+  /* The raw text of the argument of this predicate. */
+  char *arg_text;
   
   /* Information needed by the predicate processor.
      Next to each member are listed the predicates that use it. */
@@ -403,6 +431,7 @@ dev_t * get_mounted_devices PARAMS((size_t *));
 enum arg_type
   {
     ARG_OPTION,			/* regular options like -maxdepth */
+    ARG_NOOP,			/* does nothing, returns true, internal use only */
     ARG_POSITIONAL_OPTION,	/* options whose position is important (-follow) */
     ARG_TEST,			/* a like -name */
     ARG_PUNCTUATION,		/* like -o or ( */
@@ -502,35 +531,48 @@ char *find_pred_name PARAMS((PRED_FUNC pred_func));
 
 
 
-#ifdef DEBUG
+void print_predicate PARAMS((FILE *fp, const struct predicate *p));
 void print_tree PARAMS((FILE*, struct predicate *node, int indent));
 void print_list PARAMS((FILE*, struct predicate *node));
-void print_optlist PARAMS((FILE *fp, struct predicate *node));
-#endif /* DEBUG */
+void print_optlist PARAMS((FILE *fp, const struct predicate *node));
+
 
 /* tree.c */
-struct predicate *
-get_expr PARAMS((struct predicate **input, short int prev_prec));
-boolean opt_expr PARAMS((struct predicate **eval_treep));
-boolean mark_stat PARAMS((struct predicate *tree));
-boolean mark_type PARAMS((struct predicate *tree));
-
-/* util.c */
+struct predicate * build_expression_tree PARAMS((int argc, char *argv[], int end_of_leading_options));
+struct predicate * get_eval_tree PARAMS((void));
 struct predicate *get_new_pred PARAMS((const struct parser_table *entry));
 struct predicate *get_new_pred_chk_op PARAMS((const struct parser_table *entry));
+float  calculate_derived_rates PARAMS((struct predicate *p));
+
+/* util.c */
 struct predicate *insert_primary PARAMS((const struct parser_table *entry));
 struct predicate *insert_primary_withpred PARAMS((const struct parser_table *entry, PRED_FUNC fptr));
-void usage PARAMS((char *msg));
+void usage PARAMS((FILE *fp, int status, char *msg));
 extern boolean check_nofollow(void);
-extern void complete_pending_execs(struct predicate *p);
-extern void complete_pending_execdirs(struct predicate *p);
+void complete_pending_execs(struct predicate *p);
+void complete_pending_execdirs(struct predicate *p);
+int process_leading_options PARAMS((int argc, char *argv[]));
+void set_option_defaults PARAMS((struct options *p));
+
+
 /* find.c. */
 int get_info PARAMS((const char *pathname, const char *name, struct stat *p, struct predicate *pred_ptr));
-int following_links(void);
-int digest_mode(mode_t mode, const char *pathname, const char *name, struct stat *pstat, boolean leaf);
-boolean default_prints (struct predicate *pred);
+int following_links PARAMS((void));
+int digest_mode PARAMS((mode_t mode, const char *pathname, const char *name, struct stat *pstat, boolean leaf));
+boolean default_prints PARAMS((struct predicate *pred));
+boolean looks_like_expression PARAMS((const char *arg, boolean leading));
 
 
+enum DebugOption
+  {
+    DebugNone             = 0,
+    DebugExpressionTree   = 1,
+    DebugStat             = 2,
+    DebugSearch           = 4,
+    DebugTreeOpt          = 8,
+    DebugHelp             = 16
+  };
+
 struct options
 {
   /* If true, process directory before contents.  True unless -depth given. */
@@ -562,6 +604,7 @@ struct options
   /* If true, we issue warning messages
    */
   boolean warnings;
+  
   time_t start_time;		/* Time at start of execution.  */
   
   /* Seconds between 00:00 1/1/70 and either one day before now
@@ -572,6 +615,9 @@ struct options
   boolean full_days;
   
   int output_block_size;	/* Output block size.  */
+
+  /* bitmask for debug options */
+  unsigned long debug_options;
   
   enum SymlinkOption symlink_handling;
   
@@ -590,6 +636,10 @@ struct options
    * can be changed with the positional option, -regextype.
    */
   int regex_options;
+
+  /* Optimisation level.  One is the default. 
+   */
+  unsigned short optimisation_level;
 };
 extern struct options options;
 
@@ -625,10 +675,8 @@ struct state
 extern struct state state;
 extern char const *starting_dir;
 extern int starting_desc;
-extern struct predicate *eval_tree;
 extern char *program_name;
-extern struct predicate *predicates;
-extern struct predicate *last_pred;
+
 
 
 #endif
