@@ -1,19 +1,21 @@
 /* locate -- search databases for filenames that match patterns
    Copyright (C) 1994, 1996, 1998, 1999, 2000, 2003,
-                 2004, 2005, 2007 Free Software Foundation, Inc.
+                 2004, 2005 Free Software Foundation, Inc.
 
-   This program is free software: you can redistribute it and/or modify
+   This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation, either version 3 of the License, or
-   (at your option) any later version.
-   
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+   USA.
 */
 
 /* Usage: locate [options] pattern...
@@ -28,8 +30,8 @@
 
    0-28		likeliest differential counts + offset (14) to make nonnegative
    30		escape code for out-of-range count to follow in next halfword
-   128-255	bigram codes (the 128 most common, as determined by `updatedb')
-   32-127	single character (printable) ASCII remainder
+   128-255 	bigram codes (the 128 most common, as determined by `updatedb')
+   32-127  	single character (printable) ASCII remainder
 
    Earlier versions of GNU locate used to use a novel two-tiered
    string search technique, which was described in Usenix ;login:, Vol
@@ -121,9 +123,10 @@ extern int errno;
 #endif
 
 #include "locatedb.h"
-#include "xalloc.h"
-#include "error.h"
-#include "human.h"
+#include <getline.h>
+#include "../gnulib/lib/xalloc.h"
+#include "../gnulib/lib/error.h"
+#include "../gnulib/lib/human.h"
 #include "dirname.h"
 #include "closeout.h"
 #include "nextelem.h"
@@ -132,7 +135,7 @@ extern int errno;
 #include "quotearg.h"
 #include "printquoted.h"
 #include "regextype.h"
-#include "gnulib-version.h"
+
 
 /* Note that this evaluates C many times.  */
 #ifdef _LIBC
@@ -226,14 +229,15 @@ locate_read_str(char **buf, size_t *siz, FILE *fp, int delimiter, int offs)
 {
   char * p = NULL;
   size_t sz = 0;
-  int needed, nread;
+  int nread;
+  size_t needed;
 
   nread = getdelim(&p, &sz, delimiter, fp);
   if (nread >= 0)
     {
       assert(p != NULL);
       
-      needed = offs + nread + 1;
+      needed = offs + nread + 1u;
       if (needed > (*siz))
 	{
 	  char *pnew = realloc(*buf, needed);
@@ -313,7 +317,7 @@ struct process_data
   int len;
   char *original_filename;	/* The current input database entry. */
   size_t pathsize;		/* Amount allocated for it.  */
-  char *munged_filename;	/* path or basename(path) */
+  char *munged_filename;	/* path or base_name(path) */
   FILE *fp;			/* The pathname database.  */
   char *dbfile;			/* Its name, or "<stdin>" */
   /* for the old database format,
@@ -465,78 +469,31 @@ visit_justprint_unquoted(struct process_data *procdata, void *context)
   return VISIT_CONTINUE;
 }
 
-static void
-toolong (struct process_data *procdata)
-{
-  error (1, 0,
-	 _("locate database %s contains a "
-	   "filename longer than locate can handle"),
-	 procdata->dbfile);
-}
-
-static void
-extend (struct process_data *procdata, size_t siz1, size_t siz2)
-{
-  /* Figure out if the addition operation is safe before performing it. */
-  if (SIZE_MAX - siz1 < siz2)
-    {
-      toolong (procdata);
-    }
-  else if (procdata->pathsize < (siz1+siz2))
-    {
-      procdata->pathsize = siz1+siz2;
-      procdata->original_filename = x2nrealloc (procdata->original_filename,
-						&procdata->pathsize,
-						1);
-    }
-}
-
 static int
 visit_old_format(struct process_data *procdata, void *context)
 {
-  register size_t i;
+  register char *s;
   (void) context;
 
-  if (EOF == procdata->c)
-    return VISIT_ABORT;
-
   /* Get the offset in the path where this path info starts.  */
   if (procdata->c == LOCATEDB_OLD_ESCAPE)
     procdata->count += getw (procdata->fp) - LOCATEDB_OLD_OFFSET;
   else
     procdata->count += procdata->c - LOCATEDB_OLD_OFFSET;
-  assert(procdata->count > 0);
 
-  /* Overlay the old path with the remainder of the new.  Read 
-   * more data until we get to the next filename.
-   */
-  for (i=procdata->count;
+  /* Overlay the old path with the remainder of the new.  */
+  for (s = procdata->original_filename + procdata->count;
        (procdata->c = getc (procdata->fp)) > LOCATEDB_OLD_ESCAPE;)
-    {
-      if (EOF == procdata->c)
-	break;
-
-      if (procdata->c < 0200)
-	{
-	  /* An ordinary character. */	  
-	  extend (procdata, i, 1u);
-	  procdata->original_filename[i++] = procdata->c;
-	}
-      else
-	{
-	  /* Bigram markers have the high bit set. */
-	  extend (procdata, i, 2u);
-	  procdata->c &= 0177;
-	  procdata->original_filename[i++] = procdata->bigram1[procdata->c];
-	  procdata->original_filename[i++] = procdata->bigram2[procdata->c];
-	}
-    }
-
-  /* Consider the case where we executed the loop body zero times; we
-   * still need space for the terminating null byte. 
-   */
-  extend (procdata, i, 1u);
-  procdata->original_filename[i] = 0;
+    if (procdata->c < 0200)
+      *s++ = procdata->c;		/* An ordinary character.  */
+    else
+      {
+	/* Bigram markers have the high bit set. */
+	procdata->c &= 0177;
+	*s++ = procdata->bigram1[procdata->c];
+	*s++ = procdata->bigram2[procdata->c];
+      }
+  *s-- = '\0';
 
   procdata->munged_filename = procdata->original_filename;
   
@@ -588,7 +545,7 @@ static int
 visit_basename(struct process_data *procdata, void *context)
 {
   (void) context;
-  procdata->munged_filename = last_component (procdata->original_filename);
+  procdata->munged_filename = base_name(procdata->original_filename);
 
   return VISIT_CONTINUE;
 }
@@ -879,7 +836,7 @@ locate (int argc,
 	int regex,
 	int regex_options)
 {
-  char *pathpart;		/* A pattern to consider. */
+  char *pathpart; 		/* A pattern to consider. */
   int argn;			/* Index to current pattern in argv. */
   int need_fold;	/* Set when folding and any pattern is non-glob. */
   int nread;		     /* number of bytes read from an entry. */
@@ -1152,7 +1109,7 @@ Usage: %s [-d path | --database=path] [-e | -E | --[non-]existing]\n\
       [--limit=N | -l N] [-S | --statistics] [-0 | --null] [-c | --count]\n\
       [-P | -H | --nofollow] [-L | --follow] [-m | --mmap ] [ -s | --stdio ]\n\
       [-A | --all] [-p | --print] [-r | --regex ] [--regextype=TYPE]\n\
-      [--version] [--help]\n\
+      [-version] [--help]\n\
       pattern...\n"),
 	   program_name);
   fputs (_("\nReport bugs to <bug-findutils@gnu.org>.\n"), stream);
@@ -1194,6 +1151,7 @@ main (int argc, char **argv)
 {
   char *dbpath;
   unsigned long int found = 0uL;
+  int optc;
   int ignore_case = 0;
   int print = 0;
   int just_count = 0;
@@ -1226,115 +1184,107 @@ main (int argc, char **argv)
 
   check_existence = ACCEPT_EITHER;
 
-  for (;;)
-    {
-      int opti = -1;
-      int optc = getopt_long (argc, argv, "Abcd:eEil:prsm0SwHPL", longopts,
-			      &opti);
-      if (optc == -1)
+  while ((optc = getopt_long (argc, argv, "Abcd:eEil:prsm0SwHPL", longopts, (int *) 0)) != -1)
+    switch (optc)
+      {
+      case '0':
+	separator = 0;
+	print_quoted_filename = false; /* print filename 'raw'. */
 	break;
 
-      switch (optc)
+      case 'A':
+	op_and = 1;
+	break;
+
+      case 'b':
+	basename_only = 1;
+	break;
+
+      case 'c':
+	just_count = 1;
+	break;
+
+      case 'd':
+	dbpath = optarg;
+	break;
+
+      case 'e':
+	check_existence = ACCEPT_EXISTING;
+	break;
+
+      case 'E':
+	check_existence = ACCEPT_NON_EXISTING;
+	break;
+
+      case 'i':
+	ignore_case = 1;
+	break;
+	
+      case 'h':
+	usage (stdout);
+	return 0;
+
+      case 'p':
+	print = 1;
+	break;
+
+      case 'v':
+	printf (_("GNU locate version %s\n"), version_string);
+	return 0;
+
+      case 'w':
+	basename_only = 0;
+	break;
+
+      case 'r':
+	regex = 1;
+	break;
+
+      case REGEXTYPE_OPTION:
+	regex_options = get_regex_type(optarg);
+	break;
+	
+      case 'S':
+	stats = 1;
+	break;
+
+      case 'L':
+	follow_symlinks = 1;
+	break;
+
+	/* In find, -P and -H differ in the way they handle paths
+	 * given on the command line.  This is not relevant for
+	 * locate, but the -H option is supported because it is
+	 * probably more intuitive to do so.
+	 */
+      case 'P':
+      case 'H':
+	follow_symlinks = 0;
+	break;
+
+      case 'l':
 	{
-	case '0':
-	  separator = 0;
-	  print_quoted_filename = false; /* print filename 'raw'. */
-	  break;
-
-	case 'A':
-	  op_and = 1;
-	  break;
-
-	case 'b':
-	  basename_only = 1;
-	  break;
-
-	case 'c':
-	  just_count = 1;
-	  break;
-
-	case 'd':
-	  dbpath = optarg;
-	  break;
-
-	case 'e':
-	  check_existence = ACCEPT_EXISTING;
-	  break;
-
-	case 'E':
-	  check_existence = ACCEPT_NON_EXISTING;
-	  break;
-
-	case 'i':
-	  ignore_case = 1;
-	  break;
-
-	case 'h':
-	  usage (stdout);
-	  return 0;
-
-	case 'p':
-	  print = 1;
-	  break;
-
-	case 'v':
-	  printf (_("GNU locate version %s\n"), version_string);
-	  printf (_("Built using GNU gnulib version %s\n"), gnulib_version);
-	  return 0;
-
-	case 'w':
-	  basename_only = 0;
-	  break;
-
-	case 'r':
-	  regex = 1;
-	  break;
-
-	case REGEXTYPE_OPTION:
-	  regex_options = get_regex_type (optarg);
-	  break;
-
-	case 'S':
-	  stats = 1;
-	  break;
-
-	case 'L':
-	  follow_symlinks = 1;
-	  break;
-
-	  /* In find, -P and -H differ in the way they handle paths
-	   * given on the command line.  This is not relevant for
-	   * locate, but the -H option is supported because it is
-	   * probably more intuitive to do so.
-	   */
-	case 'P':
-	case 'H':
-	  follow_symlinks = 0;
-	  break;
-
-	case 'l':
-	  {
-	    char *end = optarg;
-	    strtol_error err = xstrtoumax (optarg, &end, 10, &limits.limit,
-					   NULL);
-	    if (LONGINT_OK != err)
-	      xstrtol_fatal (err, opti, optc, longopts, optarg);
-	    use_limit = 1;
-	  }
-	  break;
-
-	case 's':			/* use stdio */
-	case 'm':			/* use mmap  */
-	  /* These options are implemented simply for
-	   * compatibility with FreeBSD
-	   */
-	  break;
-
-	default:
-	  usage (stderr);
-	  return 1;
+	  char *end = optarg;
+	  strtol_error err = xstrtoumax(optarg, &end, 10, &limits.limit, NULL);
+	  if (LONGINT_OK != err)
+	    {
+	      STRTOL_FATAL_ERROR(optarg, _("argument to --limit"), err);
+	    }
+	  use_limit = 1;
 	}
-    }
+	break;
+
+      case 's':			/* use stdio */
+      case 'm':			/* use mmap  */
+	/* These options are implemented simply for
+	 * compatibility with FreeBSD
+	 */ 
+	break;
+
+      default:
+	usage (stderr);
+	return 1;
+      }
 
   if (!just_count && !stats)
     print = 1;
@@ -1342,7 +1292,7 @@ main (int argc, char **argv)
   if (stats)
     {
       if (optind == argc)
-	use_limit = 0;
+ 	  use_limit = 0;
     }
   else
     {
